@@ -16,11 +16,11 @@
 
 # pylint: disable-msg=C0301
 
-prefix = '/home/tomeu/sugar-jhbuild/install'
-data_path = '/home/tomeu/sugar-jhbuild/install/share/sugar/data'
-shell_path = '/home/tomeu/sugar-jhbuild/install/share/sugar/shell'
-locale_path = '/home/tomeu/sugar-jhbuild/install/share/locale'
-ext_path = '/home/tomeu/sugar-jhbuild/install/share/sugar/extensions'
-activities_path = "/home/tomeu/sugar-jhbuild/install/share/sugar/activities"
-version = '0.84.2'
+prefix = '/usr'
+data_path = '/usr/share/sugar/data'
+shell_path = '/usr/share/sugar/shell'
+locale_path = '/usr/share/locale'
+ext_path = '/usr/share/sugar/extensions'
+activities_path = "/usr/share/sugar/activities"
+version = '0.88.1'
 
