@@ -1,5 +1,6 @@
 # Copyright (C) 2006-2007 Red Hat, Inc.
 # Copyright (C) 2008 One Laptop Per Child
+# Copyright (C) 2010 Plan Ceibal <comunidad@plan.ceibal.edu.uy>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +24,6 @@ import gobject
 import gconf
 import gtk
 import hippo
-import dbus
 
 from sugar.graphics import style
 from sugar.graphics.icon import Icon, CanvasIcon
@@ -34,6 +34,7 @@ from sugar.activity import activityfactory
 from sugar.activity.activityhandle import ActivityHandle
 from sugar.presence import presenceservice
 from sugar import dispatch
+from sugar.datastore import datastore
 
 from jarabe.view.palettes import JournalPalette
 from jarabe.view.palettes import CurrentActivityPalette, ActivityPalette
@@ -42,7 +43,8 @@ from jarabe.view import launcher
 from jarabe.model.buddy import BuddyModel
 from jarabe.model import shell
 from jarabe.model import bundleregistry
-from jarabe import journal
+from jarabe.journal.journalactivity import get_journal
+from jarabe.journal import misc
 
 from jarabe.desktop import schoolserver
 from jarabe.desktop.schoolserver import RegisterError
@@ -65,11 +67,6 @@ about the layout can be accessed with fields of the class."""
 class FavoritesView(hippo.Canvas):
     __gtype_name__ = 'SugarFavoritesView'
 
-    __gsignals__ = {
-        'erase-activated' : (gobject.SIGNAL_RUN_FIRST,
-                             gobject.TYPE_NONE, ([str]))
-    }
-
     def __init__(self, **kwargs):
         logging.debug('STARTUP: Loading the favorites view')
 
@@ -96,7 +93,7 @@ class FavoritesView(hippo.Canvas):
 
         self._layout = None
         self._alert = None
-        self._datastore_listener = DatastoreListener()
+        self._resume_mode = True
 
         # More DND stuff
         self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
@@ -116,7 +113,7 @@ class FavoritesView(hippo.Canvas):
 
     def __settings_changed_cb(self, **kwargs):
         favorites_settings = get_settings()
-        self._set_layout(favorites_settings.layout)        
+        self._set_layout(favorites_settings.layout)
 
     def __connect_to_bundle_registry_cb(self):
         registry = bundleregistry.get_registry()
@@ -133,15 +130,12 @@ class FavoritesView(hippo.Canvas):
     def _add_activity(self, activity_info):
         if activity_info.get_bundle_id() == 'org.laptop.JournalActivity':
             return
-        icon = ActivityIcon(activity_info, self._datastore_listener)
-        icon.connect('erase-activated', self.__erase_activated_cb)
+        icon = ActivityIcon(activity_info)
         icon.props.size = style.STANDARD_ICON_SIZE
+        icon.set_resume_mode(self._resume_mode)
         self._box.insert_sorted(icon, 0, self._layout.compare_activities)
         self._layout.append(icon)
 
-    def __erase_activated_cb(self, activity_icon, bundle_id):
-        self.emit('erase-activated', bundle_id)
-
     def __activity_added_cb(self, activity_registry, activity_info):
         registry = bundleregistry.get_registry()
         if registry.is_bundle_favorite(activity_info.get_bundle_id(),
@@ -176,7 +170,7 @@ class FavoritesView(hippo.Canvas):
             self._add_activity(activity_info)
 
     def do_size_allocate(self, allocation):
-        width = allocation.width        
+        width = allocation.width
         height = allocation.height
 
         min_w_, my_icon_width = self._my_icon.get_width_request()
@@ -220,7 +214,7 @@ class FavoritesView(hippo.Canvas):
     def __motion_notify_event_cb(self, widget, event):
         if not self._pressed_button:
             return False
-        
+
         # if the mouse button is not pressed, no drag should occurr
         if not event.state & gtk.gdk.BUTTON1_MASK:
             self._pressed_button = None
@@ -246,7 +240,7 @@ class FavoritesView(hippo.Canvas):
         icon_file_name = self._last_clicked_icon.props.file_name
         # TODO: we should get the pixbuf from the widget, so it has colors, etc
         pixbuf = gtk.gdk.pixbuf_new_from_file(icon_file_name)
-        
+
         self._hot_x = pixbuf.props.width / 2
         self._hot_y = pixbuf.props.height / 2
         context.set_icon_pixbuf(pixbuf, self._hot_x, self._hot_y)
@@ -282,7 +276,9 @@ class FavoritesView(hippo.Canvas):
 
     def _set_layout(self, layout):
         if layout not in LAYOUT_MAP:
-            raise ValueError('Unknown favorites layout: %r' % layout)
+            logging.warn('Unknown favorites layout: %r' % layout)
+            layout = favoriteslayout.RingLayout.key
+            assert layout in LAYOUT_MAP
 
         if type(self._layout) == LAYOUT_MAP[layout]:
             return
@@ -329,98 +325,49 @@ class FavoritesView(hippo.Canvas):
         except RegisterError, e:
             alert.props.title = _('Registration Failed')
             alert.props.msg = _('%s') % e
-        else:    
+        else:
             alert.props.title = _('Registration Successful')
             alert.props.msg = _('You are now registered ' \
                                 'with your school server.')
-            self._my_icon.remove_register_menu()
+            self._my_icon.change_register_menu()
+            get_journal()._volumes_toolbar.setup_xs_button()
 
         ok_icon = Icon(icon_name='dialog-ok')
         alert.add_button(gtk.RESPONSE_OK, _('Ok'), ok_icon)
 
         self.add_alert(alert)
-        alert.connect('response', self.__register_alert_response_cb)            
-            
+        alert.connect('response', self.__register_alert_response_cb)
+
     def __register_alert_response_cb(self, alert, response_id):
         self.remove_alert()
 
-DS_DBUS_SERVICE = 'org.laptop.sugar.DataStore'
-DS_DBUS_INTERFACE = 'org.laptop.sugar.DataStore'
-DS_DBUS_PATH = '/org/laptop/sugar/DataStore'
+    def set_resume_mode(self, resume_mode):
+        self._resume_mode = resume_mode
+        for icon in self._box.get_children():
+            if hasattr(icon, 'set_resume_mode'):
+                icon.set_resume_mode(self._resume_mode)
 
-class DatastoreListener(object):
-    def __init__(self):
-        bus = dbus.SessionBus()
-        remote_object = bus.get_object(DS_DBUS_SERVICE, DS_DBUS_PATH)
-        self._datastore = dbus.Interface(remote_object, DS_DBUS_INTERFACE)
-        self._datastore.connect_to_signal('Created',
-                                          self.__datastore_created_cb)
-        self._datastore.connect_to_signal('Updated',
-                                          self.__datastore_updated_cb)
-        self._datastore.connect_to_signal('Deleted',
-                                          self.__datastore_deleted_cb)
-
-        self.updated = dispatch.Signal()
-        self.deleted = dispatch.Signal()
-
-    def __datastore_created_cb(self, object_id):
-        metadata = self._datastore.get_properties(object_id, byte_arrays=True)
-        self.updated.send(self, metadata=metadata)
-
-    def __datastore_updated_cb(self, object_id):
-        metadata = self._datastore.get_properties(object_id, byte_arrays=True)
-        self.updated.send(self, metadata=metadata)
-
-    def __datastore_deleted_cb(self, object_id):
-        self.deleted.send(self, object_id=object_id)
-
-    def get_last_activity_async(self, bundle_id, properties, callback_cb):
-        query = {'activity': bundle_id,
-                 'limit': 5,
-                 'order_by': ['-mtime']}
-
-        reply_handler = lambda entries, total_count: self.__reply_handler_cb(
-                entries, total_count, callback_cb)
-
-        error_handler = lambda error: self.__error_handler_cb(
-                error, callback_cb)
-
-        self._datastore.find(query, properties, byte_arrays=True,
-                               reply_handler=reply_handler,
-                               error_handler=error_handler)
-
-    def __reply_handler_cb(self, entries, total_count, callback_cb):
-        logging.debug('__reply_handler_cb')
-        callback_cb(entries)
-
-    def __error_handler_cb(self, error, callback_cb):
-        logging.debug('__error_handler_cb')
-        callback_cb(None, error)
 
 class ActivityIcon(CanvasIcon):
     __gtype_name__ = 'SugarFavoriteActivityIcon'
 
     _BORDER_WIDTH = style.zoom(3)
+    _MAX_RESUME_ENTRIES = 5
 
-    __gsignals__ = {
-        'erase-activated' : (gobject.SIGNAL_RUN_FIRST,
-                             gobject.TYPE_NONE, ([str]))
-    }
-
-    def __init__(self, activity_info, datastore_listener):
+    def __init__(self, activity_info):
         CanvasIcon.__init__(self, cache=True,
                             file_name=activity_info.get_icon())
 
         self._activity_info = activity_info
         self._journal_entries = []
         self._hovering = False
+        self._resume_mode = True
 
         self.connect('hovering-changed', self.__hovering_changed_event_cb)
         self.connect('button-release-event', self.__button_release_event_cb)
 
-        self._datastore_listener = datastore_listener
-        datastore_listener.updated.connect(self.__datastore_listener_updated_cb)
-        datastore_listener.deleted.connect(self.__datastore_listener_deleted_cb)
+        datastore.updated.connect(self.__datastore_listener_updated_cb)
+        datastore.deleted.connect(self.__datastore_listener_deleted_cb)
 
         self._refresh()
         self._update()
@@ -429,8 +376,7 @@ class ActivityIcon(CanvasIcon):
         bundle_id = self._activity_info.get_bundle_id()
         properties = ['uid', 'title', 'icon-color', 'activity', 'activity_id',
                       'mime_type', 'mountpoint']
-        self._datastore_listener.get_last_activity_async(bundle_id, properties,
-                self.__get_last_activity_async_cb)
+        self._get_last_activity_async(bundle_id, properties)
 
     def __datastore_listener_updated_cb(self, **kwargs):
         bundle_id = self._activity_info.get_bundle_id()
@@ -443,10 +389,15 @@ class ActivityIcon(CanvasIcon):
                 self._refresh()
                 break
 
-    def __get_last_activity_async_cb(self, entries, error=None):
-        if error is not None:
-            logging.error('Error retrieving most recent activities: %r' % error)
+    def _get_last_activity_async(self, bundle_id, properties):
+        query = {'activity': bundle_id} 
+        datastore.find(query, sorting=['+timestamp'],
+                       limit=self._MAX_RESUME_ENTRIES,
+                       properties=properties,
+                       reply_handler=self.__get_last_activity_reply_handler_cb,
+                       error_handler=self.__get_last_activity_error_handler_cb)
 
+    def __get_last_activity_reply_handler_cb(self, entries, total_count):
         # If there's a problem with the DS index, we may get entries not related
         # to this activity.
         checked_entries = []
@@ -457,27 +408,30 @@ class ActivityIcon(CanvasIcon):
         self._journal_entries = checked_entries
         self._update()
 
+    def __get_last_activity_error_handler_cb(self, error):
+        logging.error('Error retrieving most recent activities: %r', error)
+
     def _update(self):
         self.palette = None
-        if not self._journal_entries:
-            self.props.stroke_color = style.COLOR_BUTTON_GREY.get_svg()
-            self.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
+        if not self._resume_mode or not self._journal_entries:
+            xo_color = XoColor('%s,%s' % (style.COLOR_DESKTOP_ICON.get_svg(),
+                                          style.COLOR_TRANSPARENT.get_svg()))
         else:
-            first_entry = self._journal_entries[0]
-            self.props.xo_color = XoColor(first_entry['icon-color'])
+            xo_color = misc.get_icon_color(self._journal_entries[0])
+        self.props.xo_color = xo_color
 
     def create_palette(self):
         palette = FavoritePalette(self._activity_info, self._journal_entries)
         palette.connect('activate', self.__palette_activate_cb)
-        palette.connect('erase-activated', self.__erase_activated_cb)
+        palette.connect('entry-activate', self.__palette_entry_activate_cb)
         return palette
 
-    def __erase_activated_cb(self, palette):
-        self.emit('erase-activated', self._activity_info.get_bundle_id())
-
     def __palette_activate_cb(self, palette):
         self._activate()
 
+    def __palette_entry_activate_cb(self, palette, metadata):
+        self._resume(metadata)
+
     def __hovering_changed_event_cb(self, icon, hovering):
         self._hovering = hovering
         self.emit_paint_needed(0, 0, -1, -1)
@@ -508,7 +462,7 @@ class ActivityIcon(CanvasIcon):
         cr.stroke()
 
     def do_get_content_height_request(self, for_width):
-        height, height = CanvasIcon.do_get_content_height_request(self, 
+        height, height = CanvasIcon.do_get_content_height_request(self,
                                                                   for_width)
         height += ActivityIcon._BORDER_WIDTH * 2
         return height, height
@@ -521,27 +475,24 @@ class ActivityIcon(CanvasIcon):
     def __button_release_event_cb(self, icon, event):
         self._activate()
 
+    def _resume(self, journal_entry):
+        if not journal_entry['activity_id']:
+            journal_entry['activity_id'] = activityfactory.create_activity_id()
+        misc.resume(journal_entry, self._activity_info.get_bundle_id())
+
     def _activate(self):
-        self.palette.popdown(immediate=True)
-        if self._journal_entries:
-            entry = self._journal_entries[0]
+        if self.palette is not None:
+            self.palette.popdown(immediate=True)
 
-            shell_model = shell.get_model()
-            activity = shell_model.get_activity_by_id(entry['activity_id'])
-            if activity:
-                activity.get_window().activate(gtk.get_current_event_time())
-                return
-
-            launcher.add_launcher(entry['activity_id'],
-                                  self._activity_info.get_icon(),
-                                  XoColor(entry.get('icon-color', '')))
-            journal.misc.resume(entry, self._activity_info.get_bundle_id())
+        if self._resume_mode and self._journal_entries:
+            self._resume(self._journal_entries[0])
         else:
             client = gconf.client_get_default()
             xo_color = XoColor(client.get_string('/desktop/sugar/user/color'))
 
             activity_id = activityfactory.create_activity_id()
             launcher.add_launcher(activity_id,
+                                  self._activity_info.get_bundle_id(),
                                   self._activity_info.get_icon(),
                                   xo_color)
 
@@ -565,20 +516,29 @@ class ActivityIcon(CanvasIcon):
         return registry.get_bundle_position(self.bundle_id, self.version)
     fixed_position = property(_get_fixed_position, None)
 
+    def set_resume_mode(self, resume_mode):
+        self._resume_mode = resume_mode
+        self._update()
+
 class FavoritePalette(ActivityPalette):
     __gtype_name__ = 'SugarFavoritePalette'
 
+    __gsignals__ = {
+        'entry-activate': (gobject.SIGNAL_RUN_FIRST,
+                           gobject.TYPE_NONE, ([object])),
+    }
+
     def __init__(self, activity_info, journal_entries):
         ActivityPalette.__init__(self, activity_info)
 
-        if journal_entries and journal_entries[0].get('icon-color', ''):
-            color = XoColor(journal_entries[0]['icon-color'])
+        if not journal_entries:
+            xo_color = XoColor('%s,%s' % (style.COLOR_WHITE.get_svg(),
+                                          style.COLOR_TRANSPARENT.get_svg()))
         else:
-            color = XoColor('%s,%s' % (style.COLOR_BUTTON_GREY.get_svg(),
-                                       style.COLOR_WHITE.get_svg()))
+            xo_color = misc.get_icon_color(journal_entries[0])
 
         self.props.icon = Icon(file=activity_info.get_icon(),
-                               xo_color=color,
+                               xo_color=xo_color,
                                icon_size=gtk.ICON_SIZE_LARGE_TOOLBAR)
 
         if journal_entries:
@@ -586,8 +546,8 @@ class FavoritePalette(ActivityPalette):
 
             menu_items = []
             for entry in journal_entries:
-                icon_file_name = journal.misc.get_icon_name(entry)
-                color = XoColor(entry.get('icon-color', None))
+                icon_file_name = misc.get_icon_name(entry)
+                color = misc.get_icon_color(entry)
 
                 menu_item = MenuItem(text_label=entry['title'],
                                      file_name=icon_file_name,
@@ -606,7 +566,7 @@ class FavoritePalette(ActivityPalette):
 
     def __resume_entry_cb(self, menu_item, entry):
         if entry is not None:
-            journal.misc.resume(entry, self._bundle_id)
+            self.emit('entry-activate', entry)
 
 class CurrentActivityIcon(CanvasIcon, hippo.CanvasItem):
     def __init__(self):
@@ -623,7 +583,8 @@ class CurrentActivityIcon(CanvasIcon, hippo.CanvasItem):
         self.connect('button-release-event', self.__button_release_event_cb)
 
     def __button_release_event_cb(self, icon, event):
-        self._home_model.get_active_activity().get_window().activate(1)
+        window = self._home_model.get_active_activity().get_window()
+        window.activate(gtk.get_current_event_time())
 
     def _update(self):
         self.props.file_name = self._home_activity.get_icon_path()
@@ -670,11 +631,15 @@ class _MyIcon(MyIcon):
 
         client = gconf.client_get_default()
         backup_url = client.get_string('/desktop/sugar/backup_url')
+
         if not backup_url:
             self._register_menu = MenuItem(_('Register'), 'media-record')
-            self._register_menu.connect('activate', self.__register_activate_cb)
-            palette.menu.append(self._register_menu)
-            self._register_menu.show()
+        else:
+            self._register_menu = MenuItem(_('Register Again'), 'media-record')
+
+        self._register_menu.connect('activate', self.__register_activate_cb)
+        palette.menu.append(self._register_menu)
+        self._register_menu.show()
 
         return palette
 
@@ -684,17 +649,21 @@ class _MyIcon(MyIcon):
     def __register_activate_cb(self, menuitem):
         self.emit('register-activate')
 
-    def remove_register_menu(self):
-        self.palette.remove(self._register_menu)
+    def change_register_menu(self):
+        self.palette.menu.remove(self._register_menu)
+        self._register_menu = MenuItem(_('Register Again'), 'media-record')
+        self._register_menu.connect('activate', self.__register_activate_cb)
+        self.palette.menu.append(self._register_menu)
+        self._register_menu.show()
 
 class FavoritesSetting(object):
 
     _FAVORITES_KEY = "/desktop/sugar/desktop/favorites_layout"
 
     def __init__(self):
-        client = gconf.client_get_default() 
+        client = gconf.client_get_default()
         self._layout = client.get_string(self._FAVORITES_KEY)
-        logging.debug('FavoritesSetting layout %r' % (self._layout))
+        logging.debug('FavoritesSetting layout %r', self._layout)
 
         self._mode = None
 
@@ -704,7 +673,7 @@ class FavoritesSetting(object):
         return self._layout
 
     def set_layout(self, layout):
-        logging.debug('set_layout %r %r' % (layout, self._layout))
+        logging.debug('set_layout %r %r', layout, self._layout)
         if layout != self._layout:
             self._layout = layout
 
@@ -722,4 +691,3 @@ def get_settings():
     if _favorites_settings is None:
         _favorites_settings = FavoritesSetting()
     return _favorites_settings
-
