@@ -22,7 +22,6 @@ import os
 
 import gobject
 import gconf
-import dbus
 import gio
 import gtk
 
@@ -36,7 +35,9 @@ from sugar.graphics.palette import Palette, WidgetInvoker
 from sugar.graphics.menuitem import MenuItem
 from sugar.activity.activityhandle import ActivityHandle
 from sugar.activity import activityfactory
+from sugar.datastore import datastore
 from sugar import mime
+from sugar import env
 
 from jarabe.model import shell
 from jarabe.model import neighborhood
@@ -50,9 +51,6 @@ from jarabe.frame.frameinvoker import FrameWidgetInvoker
 from jarabe.frame.notification import NotificationIcon
 import jarabe.frame
 
-DS_DBUS_SERVICE = "org.laptop.sugar.DataStore"
-DS_DBUS_INTERFACE = "org.laptop.sugar.DataStore"
-DS_DBUS_PATH = "/org/laptop/sugar/DataStore"
 
 class ActivityButton(RadioToolButton):
     def __init__(self, home_activity, group):
@@ -61,6 +59,7 @@ class ActivityButton(RadioToolButton):
         self.set_palette_invoker(FrameWidgetInvoker(self))
 
         self._home_activity = home_activity
+        self._notify_launch_hid = None
 
         self._icon = PulsingIcon()
         self._icon.props.base_color = home_activity.get_icon_color()
@@ -74,13 +73,12 @@ class ActivityButton(RadioToolButton):
         self.set_icon_widget(self._icon)
         self._icon.show()
 
-        if home_activity.props.launching:
+        if home_activity.props.launch_status == shell.Activity.LAUNCHING:
             self._icon.props.pulsing = True
-            self._notify_launching_hid = home_activity.connect( \
-                    'notify::launching', self.__notify_launching_cb)
-        else:
-            self._notify_launching_hid = None
-            self._notif_icon = None
+            self._notify_launch_hid = home_activity.connect( \
+                    'notify::launch-status', self.__notify_launch_status_cb)
+        elif home_activity.props.launch_status == shell.Activity.LAUNCH_FAILED:
+            self._on_failed_launch()
 
     def create_palette(self):
         if self._home_activity.is_journal():
@@ -90,10 +88,18 @@ class ActivityButton(RadioToolButton):
         palette.set_group_id('frame')
         self.set_palette(palette)
 
-    def __notify_launching_cb(self, home_activity, pspec):
-        if not home_activity.props.launching:
+    def _on_failed_launch(self):
+        # TODO http://bugs.sugarlabs.org/ticket/2007
+        pass
+
+    def __notify_launch_status_cb(self, home_activity, pspec):
+        home_activity.disconnect(self._notify_launch_hid)
+        self._notify_launch_hid = None
+        if home_activity.props.launch_status == shell.Activity.LAUNCH_FAILED:
+            self._on_failed_launch()
+        else:
             self._icon.props.pulsing = False
-            home_activity.disconnect(self._notify_launching_hid)
+
 
 class BaseInviteButton(ToolButton):
     def __init__(self, invite):
@@ -177,6 +183,7 @@ class ActivityInviteButton(BaseInviteButton):
         bundle = registry.get_bundle(self._bundle_id)
 
         launcher.add_launcher(self._activity_model.get_id(),
+                              bundle.get_bundle_id(),
                               bundle.get_icon(),
                               self._activity_model.get_color())
 
@@ -335,7 +342,7 @@ class ActivitiesTray(HTray):
         filetransfer.new_file_transfer.connect(self.__new_file_transfer_cb)
 
     def __activity_added_cb(self, home_model, home_activity):
-        logging.debug('__activity_added_cb: %r' % home_activity)
+        logging.debug('__activity_added_cb: %r', home_activity)
         if self.get_children():
             group = self.get_children()[0]
         else:
@@ -343,18 +350,18 @@ class ActivitiesTray(HTray):
 
         button = ActivityButton(home_activity, group)
         self.add_item(button)
-        self._buttons[home_activity.get_activity_id()] = button
+        self._buttons[home_activity] = button
         button.connect('clicked', self.__activity_clicked_cb, home_activity)
         button.show()
 
     def __activity_removed_cb(self, home_model, home_activity):
-        logging.debug('__activity_removed_cb: %r' % home_activity)
-        button = self._buttons[home_activity.get_activity_id()]
+        logging.debug('__activity_removed_cb: %r', home_activity)
+        button = self._buttons[home_activity]
         self.remove_item(button)
-        del self._buttons[home_activity.get_activity_id()]
+        del self._buttons[home_activity]
 
     def _activate_activity(self, home_activity):
-        button = self._buttons[home_activity.get_activity_id()]
+        button = self._buttons[home_activity]
         self._freeze_button_clicks = True
         button.props.active = True
         self._freeze_button_clicks = False
@@ -366,14 +373,14 @@ class ActivitiesTray(HTray):
             self.window.process_updates(True)
 
     def __activity_changed_cb(self, home_model, home_activity):
-        logging.debug('__activity_changed_cb: %r' % home_activity)
+        logging.debug('__activity_changed_cb: %r', home_activity)
 
         # Only select the new activity, if there is no tabbing activity.
         if home_model.get_tabbing_activity() is None:
             self._activate_activity(home_activity)
 
     def __tabbing_activity_changed_cb(self, home_model, home_activity):
-        logging.debug('__tabbing_activity_changed_cb: %r' % home_activity)
+        logging.debug('__tabbing_activity_changed_cb: %r', home_activity)
         # If the tabbing_activity is set to None just do nothing.
         # The active activity will be updated a bit later (and it will
         # be set to the activity that is currently selected).
@@ -424,7 +431,7 @@ class ActivitiesTray(HTray):
 
     def __new_file_transfer_cb(self, **kwargs):
         file_transfer = kwargs['file_transfer']
-        logging.debug('__new_file_transfer_cb %r' % file_transfer)
+        logging.debug('__new_file_transfer_cb %r', file_transfer)
 
         if isinstance(file_transfer, filetransfer.IncomingFileTransfer):
             button = IncomingTransferButton(file_transfer)
@@ -463,8 +470,8 @@ class BaseTransferButton(ToolButton):
         self.props.parent.remove(self)
 
     def __notify_state_cb(self, file_transfer, pspec):
-        logging.debug('_update state: %r %r' % (file_transfer.props.state,
-                file_transfer.reason_last_change))
+        logging.debug('_update state: %r %r', file_transfer.props.state,
+                file_transfer.reason_last_change)
         if file_transfer.props.state == filetransfer.FT_STATE_CANCELLED:
             if file_transfer.reason_last_change == \
                filetransfer.FT_REASON_LOCAL_STOPPED:
@@ -476,8 +483,7 @@ class IncomingTransferButton(BaseTransferButton):
     def __init__(self, file_transfer):
         BaseTransferButton.__init__(self, file_transfer)
 
-        self._object_id = None
-        self._metadata = {}
+        self._ds_object = datastore.create()
 
         file_transfer.connect('notify::state', self.__notify_state_cb)
         file_transfer.connect('notify::transferred-bytes',
@@ -511,64 +517,42 @@ class IncomingTransferButton(BaseTransferButton):
     def __notify_state_cb(self, file_transfer, pspec):
         if file_transfer.props.state == filetransfer.FT_STATE_OPEN:
             logging.debug('__notify_state_cb OPEN')
-            self._metadata['title'] = file_transfer.title
-            self._metadata['description'] = file_transfer.description
-            self._metadata['progress'] = '0'
-            self._metadata['keep'] = '0'
-            self._metadata['buddies'] = ''
-            self._metadata['preview'] = ''
-            self._metadata['icon-color'] = file_transfer.buddy.props.color
-            self._metadata['mime_type'] = file_transfer.mime_type
-
-            datastore = self._get_datastore()
-            file_path = ''
-            transfer_ownership = True
-            self._object_id = datastore.create(self._metadata, file_path,
-                                               transfer_ownership)
-
+            self._ds_object.metadata['title'] = file_transfer.title
+            self._ds_object.metadata['description'] = file_transfer.description
+            self._ds_object.metadata['progress'] = '0'
+            self._ds_object.metadata['keep'] = '0'
+            self._ds_object.metadata['buddies'] = ''
+            self._ds_object.metadata['preview'] = ''
+            self._ds_object.metadata['icon-color'] = \
+                    file_transfer.buddy.props.color
+            self._ds_object.metadata['mime_type'] = file_transfer.mime_type
         elif file_transfer.props.state == filetransfer.FT_STATE_COMPLETED:
             logging.debug('__notify_state_cb COMPLETED')
-            self._metadata['progress'] = '100'
-
-            datastore = self._get_datastore()
-            file_path = file_transfer.destination_path
-            transfer_ownership = True
-            datastore.update(self._object_id, self._metadata, file_path,
-                             transfer_ownership,
-                             reply_handler=self.__reply_handler_cb,
-                             error_handler=self.__error_handler_cb)
-
+            self._ds_object.metadata['progress'] = '100'
+            self._ds_object.file_path = file_transfer.destination_path
+            datastore.write(self._ds_object, transfer_ownership=True,
+                            reply_handler=self.__reply_handler_cb,
+                            error_handler=self.__error_handler_cb)
         elif file_transfer.props.state == filetransfer.FT_STATE_CANCELLED:
             logging.debug('__notify_state_cb CANCELLED')
-            if self._object_id is not None:
-                datastore.delete(self._object_id,
-                                 reply_handler=self.__reply_handler_cb,
-                                 error_handler=self.__error_handler_cb)
-                self._object_id = None
+            object_id = self._ds_object.object_id
+            if object_id is not None:
+                self._ds_object.destroy()
+                datastore.delete(object_id)
+                self._ds_object = None
 
     def __notify_transferred_bytes_cb(self, file_transfer, pspec):
         progress = file_transfer.props.transferred_bytes /      \
                    file_transfer.file_size
-        self._metadata['progress'] = str(progress * 100)
-
-        datastore = self._get_datastore()
-        file_path = ''
-        transfer_ownership = True
-        datastore.update(self._object_id, self._metadata, file_path,
-                         transfer_ownership,
-                         reply_handler=self.__reply_handler_cb,
-                         error_handler=self.__error_handler_cb)
-
-    def _get_datastore(self):
-        bus = dbus.SessionBus()
-        remote_object = bus.get_object(DS_DBUS_SERVICE, DS_DBUS_PATH)
-        return dbus.Interface(remote_object, DS_DBUS_INTERFACE)
+        self._ds_object.metadata['progress'] = str(progress * 100)
+        datastore.write(self._ds_object, update_mtime=False)
 
     def __reply_handler_cb(self):
-        logging.debug('__reply_handler_cb %r' % self._object_id)
+        logging.debug('__reply_handler_cb %r', self._ds_object.object_id)
 
     def __error_handler_cb(self, error):
-        logging.debug('__error_handler_cb %r %s' % (self._object_id, error))
+        logging.debug('__error_handler_cb %r %s', self._ds_object.object_id,
+                      error)
 
     def __dismiss_clicked_cb(self, palette):
         self.remove()
@@ -654,7 +638,7 @@ class BaseTransferPalette(Palette):
             return _('%dMB') % (size / 1048576)
 
     def update_progress(self):
-        logging.debug('update_progress: %r' %
+        logging.debug('update_progress: %r',
                       self.file_transfer.props.transferred_bytes)
 
         if self.progress_bar is None:
@@ -663,7 +647,7 @@ class BaseTransferPalette(Palette):
         self.progress_bar.props.fraction = \
                 self.file_transfer.props.transferred_bytes / \
                 float(self.file_transfer.file_size)
-        logging.debug('update_progress: %r' % self.progress_bar.props.fraction)
+        logging.debug('update_progress: %r', self.progress_bar.props.fraction)
 
         transferred = self._format_size(
                 self.file_transfer.props.transferred_bytes)
@@ -688,7 +672,7 @@ class IncomingTransferPalette(BaseTransferPalette):
         self._update()
 
     def _update(self):
-        logging.debug('_update state: %r' % self.file_transfer.props.state)
+        logging.debug('_update state: %r', self.file_transfer.props.state)
         if self.file_transfer.props.state == filetransfer.FT_STATE_PENDING:
             menu_item = MenuItem(_('Accept'), icon_name='dialog-ok')
             menu_item.connect('activate', self.__accept_activate_cb)
@@ -722,7 +706,7 @@ class IncomingTransferPalette(BaseTransferPalette):
 
             for item in self.menu.get_children():
                 self.menu.remove(item)
-            
+
             menu_item = MenuItem(_('Cancel'), icon_name='dialog-cancel')
             menu_item.connect('activate', self.__cancel_activate_cb)
             self.menu.append(menu_item)
@@ -768,8 +752,11 @@ class IncomingTransferPalette(BaseTransferPalette):
     def __accept_activate_cb(self, menu_item):
         #TODO: figure out the best place to get rid of that temp file
         extension = mime.get_primary_extension(self.file_transfer.mime_type)
+        if extension is None:
+            extension = '.bin'
         fd, file_path = tempfile.mkstemp(suffix=extension,
-                prefix=self._sanitize(self.file_transfer.title))
+                prefix=self._sanitize(self.file_transfer.title),
+                dir=os.path.join(env.get_profile_path(), 'data'))
         os.close(fd)
         os.unlink(file_path)
 
@@ -815,8 +802,9 @@ class OutgoingTransferPalette(BaseTransferPalette):
         self._update()
 
     def _update(self):
-        logging.debug('_update state: %r' % self.file_transfer.props.state)
-        if self.file_transfer.props.state == filetransfer.FT_STATE_PENDING:
+        new_state = self.file_transfer.props.state
+        logging.debug('_update state: %r', new_state)
+        if new_state == filetransfer.FT_STATE_PENDING:
 
             menu_item = MenuItem(_('Cancel'), icon_name='dialog-cancel')
             menu_item.connect('activate', self.__cancel_activate_cb)
@@ -840,12 +828,12 @@ class OutgoingTransferPalette(BaseTransferPalette):
             vbox.add(label)
             label.show()
 
-        elif self.file_transfer.props.state in \
-                [filetransfer.FT_STATE_ACCEPTED, filetransfer.FT_STATE_OPEN]:
+        elif new_state in [filetransfer.FT_STATE_ACCEPTED,
+                           filetransfer.FT_STATE_OPEN]:
 
             for item in self.menu.get_children():
                 self.menu.remove(item)
-            
+
             menu_item = MenuItem(_('Cancel'), icon_name='dialog-cancel')
             menu_item.connect('activate', self.__cancel_activate_cb)
             self.menu.append(menu_item)
@@ -865,7 +853,8 @@ class OutgoingTransferPalette(BaseTransferPalette):
 
             self.update_progress()
 
-        elif self.file_transfer.props.state == filetransfer.FT_STATE_COMPLETED:
+        elif new_state in [filetransfer.FT_STATE_COMPLETED,
+                           filetransfer.FT_STATE_CANCELLED]:
 
             for item in self.menu.get_children():
                 self.menu.remove(item)
@@ -882,4 +871,3 @@ class OutgoingTransferPalette(BaseTransferPalette):
 
     def __dismiss_activate_cb(self, menu_item):
         self.emit('dismiss-clicked')
-
