@@ -1,4 +1,5 @@
 # Copyright (C) 2007, One Laptop Per Child
+# Copyright (C) 2009, Walter Bender
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +30,7 @@ from sugar.graphics.toolbox import Toolbox
 from sugar.graphics.toolcombobox import ToolComboBox
 from sugar.graphics.toolbutton import ToolButton
 from sugar.graphics.toggletoolbutton import ToggleToolButton
+from sugar.graphics.radiotoolbutton import RadioToolButton
 from sugar.graphics.combobox import ComboBox
 from sugar.graphics.menuitem import MenuItem
 from sugar.graphics.icon import Icon
@@ -36,6 +38,7 @@ from sugar.graphics.xocolor import XoColor
 from sugar.graphics import iconentry
 from sugar.graphics import style
 from sugar import mime
+from sugar import profile
 
 from jarabe.model import bundleregistry
 from jarabe.journal import misc
@@ -55,7 +58,8 @@ _ACTION_ANYTHING = 0
 _ACTION_EVERYBODY = 0
 _ACTION_MY_FRIENDS = 1
 _ACTION_MY_CLASS = 2
-            
+
+
 class MainToolbox(Toolbox):
     def __init__(self):
         Toolbox.__init__(self)
@@ -72,8 +76,8 @@ class SearchToolbar(gtk.Toolbar):
         'query-changed': (gobject.SIGNAL_RUN_FIRST,
                           gobject.TYPE_NONE,
                           ([object]))
-    }
-    
+        }
+
     def __init__(self):
         gtk.Toolbar.__init__(self)
 
@@ -106,6 +110,15 @@ class SearchToolbar(gtk.Toolbar):
         self.insert(tool_item, -1)
         tool_item.show()
 
+        self._list_view_button = ListViewButton()
+        # TODO: Connect when Grid View is implemented
+        #self._list_view.connect('toggled', self.__view_button_toggled_cb)
+        self._list_view_button.set_active(True)
+        self.insert(self._list_view_button, -1)
+        self._list_view_button.connect('sort-property-changed',
+                                       self.__sort_changed_cb)
+        self._list_view_button.show()
+
         # TODO: enable it when the DS supports saving the buddies.
         #self._with_search_combo = self._get_with_search_combo()
         #tool_item = ToolComboBox(self._with_search_combo)
@@ -188,6 +201,14 @@ class SearchToolbar(gtk.Toolbar):
             if text:
                 query['query'] = text
 
+        property, order = self._list_view_button.get_current_sort()
+
+        if order == gtk.SORT_ASCENDING:
+            sign = '+'
+        else:
+            sign = '-'
+        query['order_by'] = [sign + property]
+
         return query
 
     def _get_date_range(self):
@@ -203,13 +224,16 @@ class SearchToolbar(gtk.Toolbar):
             date_range = (today_start - timedelta(30), right_now)
         elif self._when_search_combo.props.value == _ACTION_PAST_YEAR:
             date_range = (today_start - timedelta(356), right_now)
-        
+
         return (time.mktime(date_range[0].timetuple()),
                 time.mktime(date_range[1].timetuple()))
 
     def _combo_changed_cb(self, combo):
         self._update_if_needed()
 
+    def __sort_changed_cb(self, button):
+        self._update_if_needed()
+
     def _update_if_needed(self):
         new_query = self._build_query()
         if self._query != new_query:
@@ -256,7 +280,7 @@ class SearchToolbar(gtk.Toolbar):
                 break
 
         if what_filter_index == -1:
-            logging.warning('what_filter %r not known' % what_filter)
+            logging.warning('what_filter %r not known', what_filter)
         else:
             self._what_search_combo.set_active(what_filter_index)
 
@@ -268,17 +292,30 @@ class SearchToolbar(gtk.Toolbar):
         try:
             self._what_search_combo.remove_all()
             # TRANS: Item in a combo box that filters by entry type.
-            self._what_search_combo.append_item(_ACTION_ANYTHING, _('Anything'))
+            self._what_search_combo.append_item(_ACTION_ANYTHING,
+                                                _('Anything'))
 
             registry = bundleregistry.get_registry()
             appended_separator = False
+
+            types = mime.get_all_generic_types()
+            for generic_type in types:
+                if not appended_separator:
+                    self._what_search_combo.append_separator()
+                    appended_separator = True
+                self._what_search_combo.append_item(
+                    generic_type.type_id, generic_type.name, generic_type.icon)
+                if generic_type.type_id == current_value:
+                    current_value_index = \
+                            len(self._what_search_combo.get_model()) - 1
+
+                self._what_search_combo.set_active(current_value_index)
+
+            self._what_search_combo.append_separator()
+
             for service_name in model.get_unique_values('activity'):
                 activity_info = registry.get_bundle(service_name)
                 if not activity_info is None:
-                    if not appended_separator:
-                        self._what_search_combo.append_separator()            
-                        appended_separator = True
-
                     if os.path.exists(activity_info.get_icon()):
                         self._what_search_combo.append_item(service_name,
                                 activity_info.get_name(),
@@ -291,18 +328,6 @@ class SearchToolbar(gtk.Toolbar):
                     if service_name == current_value:
                         current_value_index = \
                                 len(self._what_search_combo.get_model()) - 1
-
-            self._what_search_combo.append_separator()
-
-            types = mime.get_all_generic_types()
-            for generic_type in types :
-                self._what_search_combo.append_item(
-                    generic_type.type_id, generic_type.name, generic_type.icon)
-                if generic_type.type_id == current_value:
-                    current_value_index = \
-                            len(self._what_search_combo.get_model()) - 1
-
-                self._what_search_combo.set_active(current_value_index)
         finally:
             self._what_search_combo.handler_unblock(
                     self._what_combo_changed_sid)
@@ -325,6 +350,11 @@ class DetailToolbox(Toolbox):
         self.entry_toolbar.show()
 
 class EntryToolbar(gtk.Toolbar):
+    __gsignals__ = {
+        'volume-error': (gobject.SIGNAL_RUN_FIRST,
+                         gobject.TYPE_NONE,
+                         ([str, str]))
+        }
     def __init__(self):
         gtk.Toolbar.__init__(self)
 
@@ -393,16 +423,42 @@ class EntryToolbar(gtk.Toolbar):
     def _resume_menu_item_activate_cb(self, menu_item, service_name):
         misc.resume(self._metadata, service_name)
 
-    def _copy_menu_item_activate_cb(self, menu_item, mount):
-        model.copy(self._metadata, mount.get_root().get_path())
+    def _copy_menu_item_activate_cb(self, menu_item, mount_point):
+        file_path = model.get_file(self._metadata['uid'])
+
+        if not file_path or not os.path.exists(file_path):
+            logging.warn('Entries without a file cannot be copied.')
+            self.emit('volume-error',
+                      _('Entries without a file cannot be copied.'),
+                      _('Warning'))
+            return
+
+        try:
+            model.copy(self._metadata, mount_point)
+        except IOError, e:
+            logging.exception('Error while copying the entry. %s', e.strerror)
+            self.emit('volume-error', 
+                      _('Error while copying the entry. %s') % e.strerror,
+                      _('Error'))
 
     def _refresh_copy_palette(self):
         palette = self._copy.get_palette()
-        
+
         for menu_item in palette.menu.get_children():
             palette.menu.remove(menu_item)
             menu_item.destroy()
 
+        if self._metadata['mountpoint'] != '/':
+            journal_item = MenuItem(_('Journal'))
+            journal_item.set_image(Icon(
+                    icon_name='activity-journal',
+                    xo_color=profile.get_color(),
+                    icon_size=gtk.ICON_SIZE_MENU))
+            journal_item.connect('activate',
+                    self._copy_menu_item_activate_cb, '/')
+            journal_item.show()
+            palette.menu.append(journal_item)
+
         volume_monitor = gio.volume_monitor_get()
         for mount in volume_monitor.get_mounts():
             if self._metadata['mountpoint'] == mount.get_root().get_path():
@@ -415,7 +471,7 @@ class EntryToolbar(gtk.Toolbar):
 
             menu_item.connect('activate',
                               self._copy_menu_item_activate_cb,
-                              mount)
+                              mount.get_root().get_path())
             palette.menu.append(menu_item)
             menu_item.show()
 
@@ -441,3 +497,55 @@ class EntryToolbar(gtk.Toolbar):
                                 activity_info.get_bundle_id())
             palette.menu.append(menu_item)
             menu_item.show()
+
+class ListViewButton(RadioToolButton):
+    __gtype_name__ = 'JournalListViewButton'
+
+    __gsignals__ = {
+        'sort-property-changed': (gobject.SIGNAL_RUN_FIRST,
+                                  gobject.TYPE_NONE,
+                                  ([])),
+    }
+
+    _SORT_OPTIONS = [
+        ('timestamp', 'view-lastedit', _('View by last edit')),
+        ('ctime', 'view-created', _('View by creation date')),
+        ('filesize', 'view-size', _('View by size')),
+    ]
+
+    def __init__(self):
+        RadioToolButton.__init__(self)
+
+        self._property = 'timestamp'
+        self._order = gtk.SORT_ASCENDING
+
+        self.props.tooltip = _('List view')
+        self.props.named_icon = 'view-list'
+        # TODO: Set accelerator when Grid View is implemented
+        #self.props.accelerator = _('<Ctrl>2')
+        self.props.group = None
+
+        for property, icon, label in self._SORT_OPTIONS:
+            button = MenuItem(icon_name=icon, text_label=label)
+            button.connect('activate',
+                           self.__sort_type_changed_cb,
+                           property,
+                           icon)
+            button.show()
+            self.props.palette.menu.insert(button, -1)
+
+    def __sort_type_changed_cb(self, widget, property, named_icon):
+        self._property = property
+        #FIXME: Implement sorting order
+        self._order = gtk.SORT_ASCENDING
+        self.emit('sort-property-changed')
+
+        self.props.named_icon = named_icon
+
+        if not self.props.active:
+            self.props.active = True
+        else:
+            self.emit('toggled')
+
+    def get_current_sort(self):
+        return (self._property, self._order)
