@@ -1,4 +1,5 @@
 # Copyright (C) 2007, One Laptop Per Child
+# Copyright (C) 2010, Paraguay Educa <tecnologia@paraguayeduca.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,25 +17,31 @@
 
 import logging
 import time
-import traceback
-import sys
 import os
 from gettext import gettext as _
 
 import gio
+import gconf
+import gtk
 
 from sugar.activity import activityfactory
 from sugar.activity.activityhandle import ActivityHandle
 from sugar.graphics.icon import get_icon_file_name
+from sugar.graphics.xocolor import XoColor
+from sugar.graphics.alert import ConfirmationAlert
 from sugar import mime
-from sugar.bundle.activitybundle import ActivityBundle
+from sugar.bundle.activitybundle import ActivityBundle, DependencyException
+from sugar.bundle.bundle import AlreadyInstalledException
+from sugar.bundle.bundle import ZipExtractException, RegistrationException
 from sugar.bundle.contentbundle import ContentBundle
-from sugar.bundle.bundle import MalformedBundleException
 from sugar import util
 
-from jarabe.model import bundleregistry
+from jarabe.view import launcher
+from jarabe.model import bundleregistry, shell
 from jarabe.journal.journalentrybundle import JournalEntryBundle
 from jarabe.journal import model
+from jarabe.journal import journalwindow
+from jarabe import frame
 
 def _get_icon_for_mime(mime_type):
     generic_types = mime.get_all_generic_types()
@@ -45,7 +52,7 @@ def _get_icon_for_mime(mime_type):
                 return file_name
 
     icons = gio.content_type_get_icon(mime_type)
-    logging.debug('icons for this file: %r' % icons.props.names)
+    logging.debug('icons for this file: %r', icons.props.names)
     for icon_name in icons.props.names:
         file_name = get_icon_file_name(icon_name)
         if file_name is not None:
@@ -69,9 +76,8 @@ def get_icon_name(metadata):
             try:
                 bundle = ActivityBundle(file_path)
                 file_name = bundle.get_icon()
-            except:
-                logging.warning('Could not read bundle:\n' + \
-                                traceback.format_exc())
+            except Exception:
+                logging.exception('Could not read bundle')
 
     if file_name is None:
         file_name = _get_icon_for_mime(metadata.get('mime_type', ''))
@@ -83,41 +89,51 @@ def get_icon_name(metadata):
 
 def get_date(metadata):
     """ Convert from a string in iso format to a more human-like format. """
-    if metadata.has_key('timestamp'):
-        timestamp = float(metadata['timestamp'])
-        return util.timestamp_to_elapsed_string(timestamp)
-    elif metadata.has_key('mtime'):
-        ti = time.strptime(metadata['mtime'], "%Y-%m-%dT%H:%M:%S")
-        return util.timestamp_to_elapsed_string(time.mktime(ti))
-    else:
-        return _('No date')
+    if 'timestamp' in metadata:
+        try:
+            timestamp = float(metadata['timestamp'])
+        except (TypeError, ValueError):
+            logging.warning('Invalid timestamp: %r', metadata['timestamp'])
+        else:
+            return util.timestamp_to_elapsed_string(timestamp)
+
+    if 'mtime' in metadata:
+        try:
+            ti = time.strptime(metadata['mtime'], '%Y-%m-%dT%H:%M:%S')
+        except (TypeError, ValueError):
+            logging.warning('Invalid mtime: %r', metadata['mtime'])
+        else:
+            return util.timestamp_to_elapsed_string(time.mktime(ti))
+
+    return _('No date')
+
 
 def get_bundle(metadata):
     try:
         if is_activity_bundle(metadata):
             file_path = util.TempFilePath(model.get_file(metadata['uid']))
             if not os.path.exists(file_path):
-                logging.warning('Invalid path: %r' % file_path)
+                logging.warning('Invalid path: %r', file_path)
                 return None
             return ActivityBundle(file_path)
 
         elif is_content_bundle(metadata):
             file_path = util.TempFilePath(model.get_file(metadata['uid']))
             if not os.path.exists(file_path):
-                logging.warning('Invalid path: %r' % file_path)
+                logging.warning('Invalid path: %r', file_path)
                 return None
             return ContentBundle(file_path)
 
         elif is_journal_bundle(metadata):
             file_path = util.TempFilePath(model.get_file(metadata['uid']))
             if not os.path.exists(file_path):
-                logging.warning('Invalid path: %r' % file_path)
+                logging.warning('Invalid path: %r', file_path)
                 return None
             return JournalEntryBundle(file_path)
         else:
             return None
-    except MalformedBundleException, e:
-        logging.warning('Incorrect bundle: %r' % e)
+    except Exception:
+        logging.exception('Incorrect bundle')
         return None
 
 def _get_activities_for_mime(mime_type):
@@ -157,21 +173,7 @@ def resume(metadata, bundle_id=None):
 
         file_path = model.get_file(metadata['uid'])
         bundle = ActivityBundle(file_path)
-        if not registry.is_installed(bundle):
-            logging.debug('Installing activity bundle')
-            registry.install(bundle)
-        else:
-            logging.debug('Upgrading activity bundle')
-            registry.upgrade(bundle)
-
-        logging.debug('activityfactory.creating bundle with id %r',
-                        bundle.get_bundle_id())
-        installed_bundle = registry.get_bundle(bundle.get_bundle_id())
-        if installed_bundle:
-            activityfactory.create(installed_bundle)
-        else:
-            logging.error('Bundle %r is not installed.',
-                          bundle.get_bundle_id())
+        install_activity(bundle, lambda: _launch_bundle(bundle))
 
     elif is_content_bundle(metadata) and bundle_id is None:
 
@@ -194,17 +196,25 @@ def resume(metadata, bundle_id=None):
         activity_bundle = registry.get_bundle(activities[0].get_bundle_id())
         activityfactory.create_with_uri(activity_bundle, bundle.get_start_uri())
     else:
+        activity_id = metadata.get('activity_id', '')
+
+        if activity_id:
+            shell_model = shell.get_model()
+            activity = shell_model.get_activity_by_id(activity_id)
+            if activity:
+                activity.get_window().activate(gtk.get_current_event_time())
+                return
+
         if bundle_id is None:
             activities = get_activities(metadata)
             if not activities:
-                logging.warning('No activity can open this object, %s.' %
+                logging.warning('No activity can open this object, %s.',
                         metadata.get('mime_type', None))
                 return
             bundle_id = activities[0].get_bundle_id()
 
         bundle = registry.get_bundle(bundle_id)
 
-        activity_id = metadata.get('activity_id', '')
 
         if metadata.get('mountpoint', '/') == '/':
             object_id = metadata['uid']
@@ -212,12 +222,71 @@ def resume(metadata, bundle_id=None):
             object_id = model.copy(metadata, '/')
 
         if activity_id:
+            launcher.add_launcher(activity_id, bundle.get_bundle_id(),
+                    bundle.get_icon(), get_icon_color(metadata))
             handle = ActivityHandle(object_id=object_id,
                                     activity_id=activity_id)
             activityfactory.create(bundle, handle)
         else:
             activityfactory.create_with_object_id(bundle, object_id)
 
+
+def install_activity(bundle, finish_cb=None):
+    try:
+        registry = bundleregistry.get_registry()
+
+        if not registry.is_installed(bundle):
+            logging.debug('Installing activity bundle')
+            registry.install(bundle)
+        else:
+            logging.debug('Upgrading activity bundle')
+            registry.upgrade(bundle)
+
+        if finish_cb is not None:
+            finish_cb()
+
+    except AlreadyInstalledException:
+        _downgrade_option_alert(bundle, finish_cb)
+    except DependencyException, error:
+        frame.get_view().add_message(
+                summary=_('Cannot install %s activity') % bundle.get_name(),
+                body=str(error))
+    except (ZipExtractException, RegistrationException):
+        frame.get_view().add_message(
+                summary=_('Cannot install activity %s') % bundle.get_name(),
+                body = _('Malformed activity bundle'))
+
+
+def _launch_bundle(bundle):
+    registry = bundleregistry.get_registry()
+    logging.debug('activityfactory.creating bundle with id %r',
+                       bundle.get_bundle_id())
+    installed_bundle = registry.get_bundle(bundle.get_bundle_id())
+    if installed_bundle:
+        activityfactory.create(installed_bundle)
+    else:
+        logging.error('Bundle %r is not installed.',
+                    bundle.get_bundle_id())
+
+def _downgrade_option_alert(bundle, finish_cb):
+    alert = ConfirmationAlert()
+    alert.props.title = _('Older Version Of %s Activity') % (bundle.get_name())
+    alert.props.msg = _('Do you want to downgrade to version %s') % \
+                       bundle.get_activity_version()
+    alert.connect('response', _downgrade_alert_response_cb, bundle, finish_cb)
+    journalwindow.get_journal_window().add_alert(alert)
+    alert.show()
+
+def _downgrade_alert_response_cb(alert, response_id, bundle, finish_cb):
+    if response_id is gtk.RESPONSE_OK:
+        journalwindow.get_journal_window().remove_alert(alert)
+        registry = bundleregistry.get_registry()
+        registry.install(bundle, force_downgrade=True)
+        if finish_cb is not None:
+            finish_cb()
+    elif response_id is gtk.RESPONSE_CANCEL:
+        journalwindow.get_journal_window().remove_alert(alert)
+
 def is_activity_bundle(metadata):
     mime_type = metadata.get('mime_type', '')
     return mime_type == ActivityBundle.MIME_TYPE or \
@@ -233,3 +302,35 @@ def is_bundle(metadata):
     return is_activity_bundle(metadata) or is_content_bundle(metadata) or \
             is_journal_bundle(metadata)
 
+def get_icon_color(metadata):
+    if metadata is None or not 'icon-color' in metadata:
+        client = gconf.client_get_default()
+        return XoColor(client.get_string('/desktop/sugar/user/color'))
+    else:
+        return XoColor(metadata['icon-color'])
+
+def get_backup_identifier():
+    serial_number = get_xo_serial()
+    if serial_number is None:
+        serial_number = get_nick()
+    return serial_number
+
+def get_xo_serial():
+    path = '/ofw/serial-number'
+
+    if os.access(path, os.R_OK) == 0:
+        return None
+
+    file_descriptor = open(path, 'r')
+    content = file_descriptor.read()
+    file_descriptor.close()
+
+    if content:
+        return content.strip()
+    else:
+        logging.error('No serial number at %s', path)
+        return None
+
+def get_nick():
+    client = gconf.client_get_default()
+    return client.get_string("/desktop/sugar/user/nick")
