@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2008, One Laptop Per Child
+# Copyright (C) 2007-2010, One Laptop per Child
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
 
 import logging
 import os
+import errno
 from datetime import datetime
 import time
 import shutil
-from stat import S_IFMT, S_IFDIR, S_IFREG
-import traceback
+from stat import S_IFLNK, S_IFMT, S_IFDIR, S_IFREG
 import re
+from operator import itemgetter
 
 import gobject
 import dbus
@@ -37,11 +38,12 @@ DS_DBUS_INTERFACE = 'org.laptop.sugar.DataStore'
 DS_DBUS_PATH = '/org/laptop/sugar/DataStore'
 
 # Properties the journal cares about.
-PROPERTIES = ['uid', 'title', 'mtime', 'timestamp', 'keep', 'buddies',
-              'icon-color', 'mime_type', 'progress', 'activity', 'mountpoint',
-              'activity_id', 'bundle_id']
+PROPERTIES = ['uid', 'title', 'mtime', 'timestamp', 'ctime', 'filesize',
+              'keep', 'buddies', 'icon-color', 'mime_type', 'progress',
+              'activity', 'mountpoint', 'activity_id', 'bundle_id']
 
-PAGES_TO_CACHE = 5
+MIN_PAGES_TO_CACHE = 3
+MAX_PAGES_TO_CACHE = 5
 
 class _Cache(object):
 
@@ -49,45 +51,34 @@ class _Cache(object):
 
     def __init__(self, entries=None):
         self._array = []
-        self._dict = {}
         if entries is not None:
             self.append_all(entries)
 
     def prepend_all(self, entries):
-        for entry in entries[::-1]:
-            self._array.insert(0, entry)
-            self._dict[entry['uid']] = entry
+        self._array[0:0] = entries
 
     def append_all(self, entries):
-        for entry in entries:
-            self._array.append(entry)
-            self._dict[entry['uid']] = entry
-    
-    def remove_all(self, entries):
-        entries = entries[:]
-        for entry in entries:
-            obj = self._dict[entry['uid']]
-            self._array.remove(obj)
-            del self._dict[entry['uid']]
+        self._array += entries
 
     def __len__(self):
         return len(self._array)
 
     def __getitem__(self, key):
-        if isinstance(key, basestring):
-            return self._dict[key]
-        else:
-            return self._array[key]
+        return self._array[key]
+
+    def __delitem__(self, key):
+        del self._array[key]
+
 
 class BaseResultSet(object):
     """Encapsulates the result of a query
     """
 
-    def __init__(self, query, cache_limit):
+    def __init__(self, query, page_size):
         self._total_count  = -1
         self._position = -1
         self._query = query
-        self._cache_limit = cache_limit
+        self._page_size = page_size
 
         self._offset = 0
         self._cache = _Cache()
@@ -104,7 +95,7 @@ class BaseResultSet(object):
     def get_length(self):
         if self._total_count == -1:
             query = self._query.copy()
-            query['limit'] = self._cache_limit
+            query['limit'] = self._page_size * MIN_PAGES_TO_CACHE
             entries, self._total_count = self.find(query)
             self._cache.append_all(entries)
             self._offset = 0
@@ -118,13 +109,8 @@ class BaseResultSet(object):
     def seek(self, position):
         self._position = position
 
-    def read(self, max_count):
-        logging.debug('ResultSet.read position: %r' % self._position)
-
-        if max_count * PAGES_TO_CACHE > self._cache_limit:
-            raise RuntimeError(
-                    'max_count (%i) too big for self._cache_limit'
-                    ' (%i).' % (max_count, self._cache_limit))
+    def read(self):
+        logging.debug('ResultSet.read position: %r', self._position)
 
         if self._position == -1:
             self.seek(0)
@@ -142,31 +128,29 @@ class BaseResultSet(object):
 
         last_cached_entry = self._offset + len(self._cache)
 
-        if (remaining_forward_entries <= 0 and
-                    remaining_backwards_entries <= 0) or \
-                max_count > self._cache_limit:
+        if remaining_forward_entries <= 0 and remaining_backwards_entries <= 0:
 
             # Total cache miss: remake it
-            offset = max(0, self._position - max_count)
-            logging.debug('remaking cache, offset: %r limit: %r' % \
-                          (offset, max_count * 2))
+            limit = self._page_size * MIN_PAGES_TO_CACHE
+            offset = max(0, self._position - limit / 2)
+            logging.debug('remaking cache, offset: %r limit: %r', offset,
+                limit)
             query = self._query.copy()
-            query['limit'] = self._cache_limit
+            query['limit'] = limit
             query['offset'] = offset
             entries, self._total_count = self.find(query)
 
-            self._cache.remove_all(self._cache)
+            del self._cache[:]
             self._cache.append_all(entries)
             self._offset = offset
-            
-        elif remaining_forward_entries < 2 * max_count and \
-             last_cached_entry < self._total_count:
+
+        elif remaining_forward_entries <= 0 and remaining_backwards_entries > 0:
 
             # Add one page to the end of cache
-            logging.debug('appending one more page, offset: %r' % \
-                          last_cached_entry)
+            logging.debug('appending one more page, offset: %r',
+                last_cached_entry)
             query = self._query.copy()
-            query['limit'] = max_count
+            query['limit'] = self._page_size
             query['offset'] = last_cached_entry
             entries, self._total_count = self.find(query)
 
@@ -174,19 +158,21 @@ class BaseResultSet(object):
             self._cache.append_all(entries)
 
             # apply the cache limit
-            objects_excess = len(self._cache) - self._cache_limit
+            cache_limit = self._page_size * MAX_PAGES_TO_CACHE
+            objects_excess = len(self._cache) - cache_limit
             if objects_excess > 0:
                 self._offset += objects_excess
-                self._cache.remove_all(self._cache[:objects_excess])
+                del self._cache[:objects_excess]
 
-        elif remaining_backwards_entries < 2 * max_count and self._offset > 0:
+        elif remaining_forward_entries > 0 and \
+                remaining_backwards_entries <= 0 and self._offset > 0:
 
             # Add one page to the beginning of cache
-            limit = min(self._offset, max_count)
-            self._offset = max(0, self._offset - max_count)
+            limit = min(self._offset, self._page_size)
+            self._offset = max(0, self._offset - limit)
 
-            logging.debug('prepending one more page, offset: %r limit: %r' % 
-                          (self._offset, limit))
+            logging.debug('prepending one more page, offset: %r limit: %r',
+                self._offset, limit)
             query = self._query.copy()
             query['limit'] = limit
             query['offset'] = self._offset
@@ -196,20 +182,19 @@ class BaseResultSet(object):
             self._cache.prepend_all(entries)
 
             # apply the cache limit
-            objects_excess = len(self._cache) - self._cache_limit
+            cache_limit = self._page_size * MAX_PAGES_TO_CACHE
+            objects_excess = len(self._cache) - cache_limit
             if objects_excess > 0:
-                self._cache.remove_all(self._cache[-objects_excess:])
+                del self._cache[-objects_excess:]
         else:
             logging.debug('cache hit and no need to grow the cache')
 
-        first_pos = self._position - self._offset
-        last_pos = self._position - self._offset + max_count
-        return self._cache[first_pos:last_pos]
+        return self._cache[self._position - self._offset]
 
 class DatastoreResultSet(BaseResultSet):
     """Encapsulates the result of a query on the datastore
     """
-    def __init__(self, query, cache_limit):
+    def __init__(self, query, page_size):
 
         if query.get('query', '') and not query['query'].startswith('"'):
             query_text = ''
@@ -222,7 +207,7 @@ class DatastoreResultSet(BaseResultSet):
 
             query['query'] = query_text
 
-        BaseResultSet.__init__(self, query, cache_limit)
+        BaseResultSet.__init__(self, query, page_size)
 
     def find(self, query):
         entries, total_count = _get_datastore().find(query, PROPERTIES,
@@ -236,11 +221,13 @@ class DatastoreResultSet(BaseResultSet):
 class InplaceResultSet(BaseResultSet):
     """Encapsulates the result of a query on a mount point
     """
-    def __init__(self, query, cache_limit, mount_point):
-        BaseResultSet.__init__(self, query, cache_limit)
+    def __init__(self, query, page_size, mount_point):
+        BaseResultSet.__init__(self, query, page_size)
         self._mount_point = mount_point
         self._file_list = None
-        self._pending_directories = 0
+        self._pending_directories = []
+        self._visited_directories = []
+        self._pending_files = []
         self._stopped = False
 
         query_text = query.get('query', '')
@@ -263,15 +250,26 @@ class InplaceResultSet(BaseResultSet):
 
         self._mime_types = query.get('mime_type', [])
 
+        self._sort = query.get('order_by', ['+timestamp'])[0]
+
     def setup(self):
         self._file_list = []
-        self._recurse_dir(self._mount_point)
+        self._pending_directories = [self._mount_point]
+        self._visited_directories = []
+        self._pending_files = []
+        gobject.idle_add(self._scan)
 
     def stop(self):
         self._stopped = True
 
     def setup_ready(self):
-        self._file_list.sort(lambda a, b: b[2] - a[2])
+        if self._sort[1:] == 'filesize':
+            keygetter = itemgetter(3)
+        else:
+            keygetter = itemgetter(2) # timestamp
+        self._file_list.sort(lambda a, b: cmp(b, a),
+                             key=keygetter,
+                             reverse=self._sort[0]=='-')
         self.ready.send(self)
 
     def find(self, query):
@@ -290,66 +288,116 @@ class InplaceResultSet(BaseResultSet):
         files = self._file_list[offset:offset + limit]
 
         entries = []
-        for file_path, stat, mtime_ in files:
+        for file_path, stat, mtime_, size_ in files:
             metadata = _get_file_metadata(file_path, stat)
             metadata['mountpoint'] = self._mount_point
             entries.append(metadata)
 
-        logging.debug('InplaceResultSet.find took %f s.' % (time.time() - t))
+        logging.debug('InplaceResultSet.find took %f s.', time.time() - t)
 
         return entries, total_count
 
-    def _recurse_dir(self, dir_path):
+    def _scan(self):
         if self._stopped:
+            return False
+
+        self.progress.send(self)
+
+        if self._pending_files:
+            self._scan_a_file()
+            return True
+
+        if self._pending_directories:
+            self._scan_a_directory()
+            return True
+
+        self.setup_ready()
+        self._visited_directories = []
+        return False
+
+    def _scan_a_file(self):
+        full_path = self._pending_files.pop(0)
+
+        try:
+            stat = os.lstat(full_path)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                logging.exception(
+                    'Error reading metadata of file %r', full_path)
             return
 
-        for entry in os.listdir(dir_path):
-            if entry.startswith('.'):
-                continue
-            full_path = dir_path + '/' + entry
+        if S_IFMT(stat.st_mode) == S_IFLNK:
+            try:
+                link = os.readlink(full_path)
+            except OSError, e:
+                logging.exception(
+                    'Error reading target of link %r', full_path)
+                return
+
+            if not os.path.abspath(link).startswith(self._mount_point):
+                return
+
             try:
                 stat = os.stat(full_path)
-                if S_IFMT(stat.st_mode) == S_IFDIR:
-                    self._pending_directories += 1
-                    gobject.idle_add(lambda s=full_path: self._recurse_dir(s))
 
-                elif S_IFMT(stat.st_mode) == S_IFREG:
-                    add_to_list = True
+            except OSError, e:
+                if e.errno != errno.ENOENT:
+                    logging.exception(
+                        'Error reading metadata of linked file %r', full_path)
+                return
 
-                    if self._regex is not None and \
-                            not self._regex.match(full_path):
-                        add_to_list = False
+        if S_IFMT(stat.st_mode) == S_IFDIR:
+            id_tuple = stat.st_ino, stat.st_dev
+            if not id_tuple in self._visited_directories:
+                self._visited_directories.append(id_tuple)
+                self._pending_directories.append(full_path)
+            return
 
-                    if None not in [self._date_start, self._date_end] and \
-                            (stat.st_mtime < self._date_start or
-                             stat.st_mtime > self._date_end):
-                        add_to_list = False
+        if S_IFMT(stat.st_mode) != S_IFREG:
+            return
 
-                    if self._mime_types:
-                        mime_type = gio.content_type_guess(filename=full_path)
-                        if mime_type not in self._mime_types:
-                            add_to_list = False
+        if self._regex is not None and \
+                not self._regex.match(full_path):
+            return
 
-                    if add_to_list:
-                        file_info = (full_path, stat, int(stat.st_mtime))
-                        self._file_list.append(file_info)
+        if self._date_start is not None and stat.st_mtime < self._date_start:
+            return
 
-                    self.progress.send(self)
+        if self._date_end is not None and stat.st_mtime > self._date_end:
+            return
 
-            except Exception:
-                logging.error('Error reading file %r: %s' % \
-                              (full_path, traceback.format_exc()))
+        if self._mime_types:
+            mime_type = gio.content_type_guess(filename=full_path)
+            if mime_type not in self._mime_types:
+                return
 
-        if self._pending_directories == 0:
-            self.setup_ready()
-        else:
-            self._pending_directories -= 1
+        file_info = (full_path, stat, int(stat.st_mtime), stat.st_size)
+        self._file_list.append(file_info)
+
+        return
+
+    def _scan_a_directory(self):
+        dir_path = self._pending_directories.pop(0)
+
+        try:
+            entries = os.listdir(dir_path)
+        except OSError, e:
+            if e.errno != errno.EACCES:
+                logging.exception('Error reading directory %r', dir_path)
+            return
+
+        for entry in entries:
+            if entry.startswith('.'):
+                continue
+            self._pending_files.append(dir_path + '/' + entry)
+        return
 
 def _get_file_metadata(path, stat):
     client = gconf.client_get_default()
     return {'uid': path,
             'title': os.path.basename(path),
             'timestamp': stat.st_mtime,
+            'filesize': stat.st_size,
             'mime_type': gio.content_type_guess(filename=path),
             'activity': '',
             'activity_id': '',
@@ -379,21 +427,19 @@ def _datastore_updated_cb(object_id):
 def _datastore_deleted_cb(object_id):
     deleted.send(None, object_id=object_id)
 
-def find(query, page_size):
+def find(query_, page_size):
     """Returns a ResultSet
     """
-    if 'order_by' not in query:
-        query['order_by'] = ['-mtime']
-    
+    query = query_.copy()
+
     mount_points = query.pop('mountpoints', ['/'])
     if mount_points is None or len(mount_points) != 1:
         raise ValueError('Exactly one mount point must be specified')
 
-    cache_limit = page_size * PAGES_TO_CACHE
     if mount_points[0] == '/':
-        return DatastoreResultSet(query, cache_limit)
+        return DatastoreResultSet(query, page_size)
     else:
-        return InplaceResultSet(query, cache_limit, mount_points[0])
+        return InplaceResultSet(query, page_size, mount_points[0])
 
 def _get_mount_point(path):
     dir_path = os.path.dirname(path)
@@ -419,16 +465,31 @@ def get_file(object_id):
     """Returns the file for an object
     """
     if os.path.exists(object_id):
-        logging.debug('get_file asked for file with path %r' % object_id)
+        logging.debug('get_file asked for file with path %r', object_id)
         return object_id
     else:
-        logging.debug('get_file asked for entry with id %r' % object_id)
+        logging.debug('get_file asked for entry with id %r', object_id)
         file_path = _get_datastore().get_filename(object_id)
         if file_path:
             return util.TempFilePath(file_path)
         else:
             return None
 
+def get_file_size(object_id):
+    """Return the file size for an object
+    """
+    logging.debug('get_file_size %r', object_id)
+    if os.path.exists(object_id):
+        return os.stat(object_id).st_size
+
+    file_path = _get_datastore().get_filename(object_id)
+    if file_path:
+        size = os.stat(file_path).st_size
+        os.remove(file_path)
+        return size
+
+    return 0
+
 def get_unique_values(key):
     """Returns a list with the different values a property has taken
     """
@@ -440,6 +501,7 @@ def delete(object_id):
     """
     if os.path.exists(object_id):
         os.unlink(object_id)
+        deleted.send(None, object_id=object_id)
     else:
         _get_datastore().delete(object_id)
 
@@ -452,13 +514,13 @@ def copy(metadata, mount_point):
     metadata['mountpoint'] = mount_point
     del metadata['uid']
 
-    return write(metadata, file_path)
+    return write(metadata, file_path, transfer_ownership=False)
 
-def write(metadata, file_path='', update_mtime=True):
+def write(metadata, file_path='', update_mtime=True, transfer_ownership=True):
     """Creates or updates an entry for that id
     """
-    logging.debug('model.write %r %r %r' % (metadata.get('uid', ''), file_path,
-                                            update_mtime))
+    logging.debug('model.write %r %r %r', metadata.get('uid', ''), file_path,
+        update_mtime)
     if update_mtime:
         metadata['mtime'] = datetime.now().isoformat()
         metadata['timestamp'] = int(time.time())
@@ -468,11 +530,11 @@ def write(metadata, file_path='', update_mtime=True):
             object_id = _get_datastore().update(metadata['uid'],
                                                  dbus.Dictionary(metadata),
                                                  file_path,
-                                                 True)
+                                                 transfer_ownership)
         else:
             object_id = _get_datastore().create(dbus.Dictionary(metadata),
                                                  file_path,
-                                                 True)
+                                                 transfer_ownership)
     else:
         if not os.path.exists(file_path):
             raise ValueError('Entries without a file cannot be copied to '
@@ -484,15 +546,18 @@ def write(metadata, file_path='', update_mtime=True):
         destination_path = os.path.join(metadata['mountpoint'], file_name)
         shutil.copy(file_path, destination_path)
         object_id = destination_path
+        created.send(None, object_id=object_id)
 
     return object_id
 
 def _get_file_name(title, mime_type):
     file_name = title
 
-    extension = '.' + mime.get_primary_extension(mime_type)
-    if not file_name.endswith(extension):
-        file_name += extension
+    extension = mime.get_primary_extension(mime_type)
+    if extension is not None and extension:
+        extension = '.' + extension
+        if not file_name.endswith(extension):
+            file_name += extension
 
     # Invalid characters in VFAT filenames. From
     # http://en.wikipedia.org/wiki/File_Allocation_Table
@@ -505,15 +570,15 @@ def _get_file_name(title, mime_type):
     max_len = 250
     if len(file_name) > max_len:
         name, extension = os.path.splitext(file_name)
-        file_name = name[0:max_len - extension] + extension
-    
+        file_name = name[0:max_len - len(extension)] + extension
+
     return file_name
 
 def _get_unique_file_name(mount_point, file_name):
     if os.path.exists(os.path.join(mount_point, file_name)):
+        name, extension = os.path.splitext(file_name)
         i = 1
         while len(file_name) <= 255:
-            name, extension = os.path.splitext(file_name)
             file_name = name + '_' + str(i) + extension
             if not os.path.exists(os.path.join(mount_point, file_name)):
                 break
@@ -521,7 +586,10 @@ def _get_unique_file_name(mount_point, file_name):
 
     return file_name
 
+def is_editable(metadata):
+    mountpoint = metadata.get('mountpoint', '/')
+    return mountpoint == '/'
+
 created = dispatch.Signal()
 updated = dispatch.Signal()
 deleted = dispatch.Signal()
-
