@@ -21,7 +21,7 @@ import traceback
 
 import gobject
 import gio
-import cjson
+import simplejson
 
 from sugar.bundle.activitybundle import ActivityBundle
 from sugar.bundle.contentbundle import ContentBundle
@@ -31,6 +31,7 @@ from sugar.bundle.bundle import MalformedBundleException, \
 from sugar import env
 
 from jarabe import config
+from jarabe.model import mimeregistry
 
 class BundleRegistry(gobject.GObject):
     """Tracks the available activity bundles"""
@@ -64,11 +65,13 @@ class BundleRegistry(gobject.GObject):
 
         self._last_defaults_mtime = -1
         self._favorite_bundles = {}
+        self._protected_activities = []
 
         try:
             self._load_favorites()
-        except Exception:
-            logging.exception('Error while loading favorite_activities.')
+            self._load_protected()
+        except Exception, e:
+            logging.exception('Error while loading activities defaults: %s' % str(e))
 
         self._merge_default_favorites()
 
@@ -107,7 +110,7 @@ class BundleRegistry(gobject.GObject):
     def _load_favorites(self):
         favorites_path = env.get_profile_path('favorite_activities')
         if os.path.exists(favorites_path):
-            favorites_data = cjson.decode(open(favorites_path).read())
+            favorites_data = simplejson.load(open(favorites_path))
 
             favorite_bundles = favorites_data['favorites']
             if not isinstance(favorite_bundles, dict):
@@ -125,6 +128,19 @@ class BundleRegistry(gobject.GObject):
             self._last_defaults_mtime = float(favorites_data['defaults-mtime'])
             self._favorite_bundles = favorite_bundles
 
+    def _load_protected(self):
+        f = open(os.path.join(config.data_path, 'activities.protected'), 'r')
+
+        for line in f.readlines():
+            line = line.strip()
+            if line and not line.startswith('#'):
+                self._protected_activities.append(line)
+
+        f.close()
+
+    def protects_activity(self, bundle_id):
+        return (bundle_id in self._protected_activities)
+
     def _merge_default_favorites(self):
         default_activities = []
         defaults_path = os.path.join(config.data_path, 'activities.defaults')
@@ -158,12 +174,22 @@ class BundleRegistry(gobject.GObject):
         self._write_favorites_file()
 
     def get_bundle(self, bundle_id):
-        """Returns an bundle given his service name"""
+        """Returns a bundle given service name or substring,
+        returns None if there is either no match, or more than one
+        match by substring."""
+        result = []
+        key = bundle_id.lower()
+
         for bundle in self._bundles:
-            if bundle.get_bundle_id() == bundle_id:
+            name = bundle.get_bundle_id()
+            if name == bundle_id:
                 return bundle
+            if key in name.lower():
+                result.append(bundle)
+        if len(result) == 1:
+            return result[0]
         return None
-    
+
     def __iter__(self):
         return self._bundles.__iter__()
 
@@ -244,13 +270,25 @@ class BundleRegistry(gobject.GObject):
 
     def get_activities_for_type(self, mime_type):
         result = []
+
+        mime = mimeregistry.get_registry()
+        default_bundle_id = mime.get_default_activity(mime_type)
+        default_bundle = None
+
         for bundle in self._bundles:
             if bundle.get_mime_types() and mime_type in bundle.get_mime_types():
-                if self.get_default_for_type(mime_type) == \
+
+                if bundle.get_bundle_id() == default_bundle_id:
+                    default_bundle = bundle
+                elif self.get_default_for_type(mime_type) == \
                         bundle.get_bundle_id():
                     result.insert(0, bundle)
                 else:
                     result.append(bundle)
+
+        if default_bundle is not None:
+            result.insert(0, default_bundle)
+
         return result
 
     def get_default_for_type(self, mime_type):
@@ -322,7 +360,7 @@ class BundleRegistry(gobject.GObject):
         path = env.get_profile_path('favorite_activities')
         favorites_data = {'defaults-mtime': self._last_defaults_mtime,
                           'favorites': self._favorite_bundles}
-        open(path, 'w').write(cjson.encode(favorites_data))
+        simplejson.dump(favorites_data, open(path, 'w'), indent=1)
 
     def is_installed(self, bundle):
         # TODO treat ContentBundle in special way
@@ -338,20 +376,29 @@ class BundleRegistry(gobject.GObject):
                 return True
         return False
 
-    def install(self, bundle):
+    def install(self, bundle, uid=None, force_downgrade=False):
+        if hasattr(bundle, 'meets_restriction'):
+            bundle.meets_restriction('sugar', config.version)
+
         activities_path = env.get_user_activities_path()
 
         for installed_bundle in self._bundles:
             if bundle.get_bundle_id() == installed_bundle.get_bundle_id() and \
                     bundle.get_activity_version() <= \
                         installed_bundle.get_activity_version():
-                raise AlreadyInstalledException
+                if not force_downgrade:
+                    raise AlreadyInstalledException
+                else:
+                    self.uninstall(installed_bundle, force=True)
             elif bundle.get_bundle_id() == installed_bundle.get_bundle_id():
                 self.uninstall(installed_bundle, force=True)
 
         install_dir = env.get_user_activities_path()
-        install_path = bundle.install(install_dir)
-        
+        if isinstance(bundle, JournalEntryBundle):
+            install_path = bundle.install(uid)
+        else:
+            install_path = bundle.install(install_dir)
+
         # TODO treat ContentBundle in special way
         # needs rethinking while fixing ContentBundle support
         if isinstance(bundle, ContentBundle) or \
@@ -360,7 +407,7 @@ class BundleRegistry(gobject.GObject):
         elif not self.add_bundle(install_path):
             raise RegistrationException
 
-    def uninstall(self, bundle, force=False):        
+    def uninstall(self, bundle, force=False, delete_profile=False):
         # TODO treat ContentBundle in special way
         # needs rethinking while fixing ContentBundle support
         if isinstance(bundle, ContentBundle) or \
@@ -383,7 +430,7 @@ class BundleRegistry(gobject.GObject):
 
         install_path = act.get_path()
 
-        bundle.uninstall(install_path, force)
+        bundle.uninstall(install_path, force, delete_profile)
 
         if not self.remove_bundle(install_path):
             raise RegistrationException
